%define gobuild(o:) %{expand:
  # https://bugzilla.redhat.com/show_bug.cgi?id=995136#c12
  %global _dwz_low_mem_die_limit 0
  %ifnarch ppc64
  go build -buildmode pie -compiler gc -tags="rpm_crashtraceback ${BUILDTAGS:-}" -ldflags "${LDFLAGS:-}%{?currentgoldflags} -B 0x$(head -c20 /dev/urandom|od -An -tx1|tr -d ' \\n') -extldflags '%__global_ldflags %{?__golang_extldflags}' -compressdwarf=false" -a -v -x %{?**};
  %else
  go build                -compiler gc -tags="rpm_crashtraceback ${BUILDTAGS:-}" -ldflags "${LDFLAGS:-}%{?currentgoldflags} -B 0x$(head -c20 /dev/urandom|od -An -tx1|tr -d ' \\n') -extldflags '%__global_ldflags %{?__golang_extldflags}' -compressdwarf=false" -a -v -x %{?**};
  %endif
}

# https://github.com/jedisct1/dnscrypt-proxy
%global goipath         github.com/jedisct1/dnscrypt-proxy
Version:                2.0.39
%global tag             2.0.39

%global common_description %{expand:
A flexible DNS proxy, with support for modern encrypted DNS protocols such as
DNSCrypt v2 and DNS-over-HTTP/2.

Features:

 - DNS traffic encryption and authentication. Supports DNS-over-HTTPS (DoH)
 and DNSCrypt.
 - DNSSEC compatible
 - DNS query monitoring, with separate log files for regular and suspicious
 queries
 - Pattern-based local blocking of DNS names and IP addresses
 - Time-based filtering, with a flexible weekly schedule
 - Transparent redirection of specific domains to specific resolvers
 - DNS caching, to reduce latency and improve privacy
 - Local IPv6 blocking to reduce latency on IPv4-only networks
 - Load balancing: pick a set of resolvers, dnscrypt-proxy will automatically
 measure and keep track of their speed, and balance the traffic across the
 fastest available ones.
 - Cloaking: like a HOSTS file on steroids, that can return preconfigured
 addresses for specific names, or resolve and return the IP address of other
 names. This can be used for local development as well as to enforce safe
 search results on Google, Yahoo and Bing.
 - Automatic background updates of resolvers lists
 - Can force outgoing connections to use TCP; useful with tunnels such as Tor.}

Name:           dnscrypt-proxy2
Release:        1%{?dist}
Summary:        Flexible DNS proxy, with support for encrypted DNS protocols

License:        ISC
URL:            https://github.com/jedisct1/dnscrypt-proxy
Source0:        %{url}/archive/%{version}/dnscrypt-proxy-%{version}.tar.gz
Source1:        dnscrypt-proxy.service
Source2:        dnscrypt-proxy.socket

# Largely inspired by Arch packaging
# https://git.archlinux.org/svntogit/community.git/tree/trunk/configuration.diff?h=packages/dnscrypt-proxy
Patch0:         dnscrypt-proxy-2.0.39-custom_config.patch

# e.g. el6 has ppc64 arch without gcc-go, so EA tag is required
ExclusiveArch:  %{?go_arches:%{go_arches}}%{!?go_arches:%{ix86} x86_64 aarch64 %{arm}}
# If go_compiler is not set to 1, there is no virtual provide. Use golang instead.
BuildRequires:  %{?go_compiler:compiler(go-compiler)}%{!?go_compiler:golang}

BuildRequires:  systemd

# For SELinux workaround
BuildRequires: selinux-policy-devel
Requires(post): policycoreutils
Requires(preun): policycoreutils
Requires(postun): policycoreutils

%{?systemd_requires}

%description
%{common_description}

%prep
%autosetup -p1 -n dnscrypt-proxy-%{version}

%build
mkdir -p ./_build/src/github.com/jedisct1
ln -s $(pwd) ./_build/src/github.com/jedisct1/dnscrypt-proxy
export GOPATH=$(pwd)/_build:%{gopath}
export GO111MODULE=off
export GOFLAGS=-mod=vendor

%gobuild -o _bin/dnscrypt-proxy %{goipath}/dnscrypt-proxy

%install
install -m 0755 -vd                          %{buildroot}%{_bindir}
install -m 0755 -vp _bin/dnscrypt-proxy      %{buildroot}%{_bindir}/
install -Dpm 0644 dnscrypt-proxy/example-dnscrypt-proxy.toml %{buildroot}%{_sysconfdir}/dnscrypt-proxy/dnscrypt-proxy.toml
install -Dpm 0644 dnscrypt-proxy/example-blacklist.txt %{buildroot}%{_sysconfdir}/dnscrypt-proxy/blacklist.txt
install -Dpm 0644 dnscrypt-proxy/example-cloaking-rules.txt %{buildroot}%{_sysconfdir}/dnscrypt-proxy/cloaking-rules.txt
install -Dpm 0644 dnscrypt-proxy/example-forwarding-rules.txt %{buildroot}%{_sysconfdir}/dnscrypt-proxy/forwarding-rules.txt
install -Dpm 0644 dnscrypt-proxy/example-whitelist.txt %{buildroot}%{_sysconfdir}/dnscrypt-proxy/whitelist.txt
install -Dpm 0644 %{S:1} %{buildroot}%{_unitdir}/dnscrypt-proxy.service
install -Dpm 0644 %{S:2} %{buildroot}%{_unitdir}/dnscrypt-proxy.socket

# Temporary SELinux workaround
# https://github.com/fedora-selinux/selinux-policy/issues/231
mkdir selinux
cd selinux

cat << EOF > my-ptproxy.te
module my-ptproxy 1.0;

require {
type var_t;
type init_t;
class dir { create setattr };
class lnk_file { create getattr read };
}

#============= init_t ==============
allow init_t var_t:dir { create setattr };
allow init_t var_t:lnk_file create;
EOF

make -f %{_datadir}/selinux/devel/Makefile
install -p -m 644 -D my-ptproxy.pp %{buildroot}%{_datadir}/selinux/packages/dnscrypt-proxy/my-ptproxy.pp

%post
%systemd_post dnscrypt-proxy.service
if [ "$1" -le "1" ] ; then # First install
semodule -i %{_datadir}/selinux/packages/dnscrypt-proxy/my-ptproxy.pp 2>/dev/null || :
fi

%preun
%systemd_preun dnscrypt-proxy.service
if [ "$1" -lt "1" ] ; then # Final removal
semodule -r my-ptproxy 2>/dev/null || :
fi

%postun
%systemd_postun dnscrypt-proxy.service
if [ "$1" -ge "1" ] ; then # Upgrade
semodule -i %{_datadir}/selinux/packages/dnscrypt-proxy/my-ptproxy.pp 2>/dev/null || :
fi

%files
%license LICENSE
%doc README.md ChangeLog
%{_bindir}/dnscrypt-proxy
%dir %{_sysconfdir}/dnscrypt-proxy
%config(noreplace) %{_sysconfdir}/dnscrypt-proxy/dnscrypt-proxy.toml
%config(noreplace) %{_sysconfdir}/dnscrypt-proxy/blacklist.txt
%config(noreplace) %{_sysconfdir}/dnscrypt-proxy/cloaking-rules.txt
%config(noreplace) %{_sysconfdir}/dnscrypt-proxy/forwarding-rules.txt
%config(noreplace) %{_sysconfdir}/dnscrypt-proxy/whitelist.txt
%{_datadir}/selinux/packages/dnscrypt-proxy/my-ptproxy.pp
%config(noreplace) %{_unitdir}/dnscrypt-proxy.socket
%{_unitdir}/dnscrypt-proxy.service

%changelog
* Mon Jan 20 2020 Robert-André Mauchin <zebob.m@gmail.com> - 2.0.39-1
- EPEL 7 build (#1792010)
